<?php

namespace Lc\ShopBundle\Services;

use App\Entity\OrderProductReductionCatalog;
use App\Entity\OrderShop;
use Doctrine\ORM\EntityManagerInterface;
use Lc\ShopBundle\Context\MerchantUtilsInterface;
use Lc\ShopBundle\Context\OrderReductionCartInterface;
use Lc\ShopBundle\Context\OrderShopInterface;
use Lc\ShopBundle\Context\OrderStatusHistoryInterface;
use Lc\ShopBundle\Context\OrderStatusInterface;
use Lc\ShopBundle\Context\ProductFamilyUtilsInterface;
use Lc\ShopBundle\Context\ReductionCartInterface;
use Lc\ShopBundle\Context\ReductionCreditInterface;
use Lc\ShopBundle\Context\UserInterface;
use Symfony\Component\Security\Core\Security;

class OrderUtils
{
        protected $em;
        protected $security;
        protected $userUtils;
        protected $merchantUtils;
        protected $orderShopRepo;
        protected $priceUtils;
        protected $productFamilyUtils;

        public function __construct(EntityManagerInterface $em, Security $security, UserUtils $userUtils,
                                    MerchantUtilsInterface $merchantUtils, PriceUtils $priceUtils, ProductFamilyUtilsInterface $productFamilyUtils)
        {
                $this->em = $em;
                $this->security = $security;
                $this->userUtils = $userUtils;
                $this->merchantUtils = $merchantUtils;
                $this->orderShopRepo = $this->em->getRepository($this->em->getClassMetadata(OrderShopInterface::class)->getName());
                $this->priceUtils = $priceUtils;
                $this->productFamilyUtils = $productFamilyUtils;
        }

        public function getCartCurrent()
        {
                $paramsSearchOrderShop = [];

                $user = $this->security->getUser();
                $visitor = $this->userUtils->getVisitorCurrent();

                if ($user) {
                        $paramsSearchOrderShop['user'] = $user;
                }

                if ($visitor) {
                        $paramsSearchOrderShop['visitor'] = $visitor;
                }

                $orderShop = $this->orderShopRepo->findCartCurrent($paramsSearchOrderShop);

                if (!$orderShop) {
                        $orderShop = $this->createOrderShop([
                                'user' => $user,
                                'visitor' => $visitor,
                                'merchant' => $this->merchantUtils->getMerchantCurrent()
                        ]);
                }

                return $orderShop;
        }

        public function createOrderShop($params)
        {
                $orderShop = new OrderShop();

                $orderShopBelongTo = false;
                if (isset($params['user']) && $params['user']) {
                        $orderShopBelongTo = true;
                        $orderShop->setUser($params['user']);
                }
                if (isset($params['visitor']) && $params['visitor']) {
                        $orderShopBelongTo = true;
                        $orderShop->setVisitor($params['visitor']);
                }

                if (!$orderShopBelongTo) {
                        throw new \ErrorException('La commande doit être liée à un utilisateur ou à un visiteur.');
                }

                if (isset($params['merchant']) && $params['merchant']) {
                        $orderShop->setMerchant($params['merchant']);
                } else {
                        throw new \ErrorException('La commande doit être liée à un merchant.');
                }

                $orderShop = $this->setOrderStatus('cart', $orderShop);

                if ($this->security->getUser()) {
                        $orderShop->setCreatedBy($this->security->getUser());
                        $orderShop->setUpdatedBy($this->security->getUser());
                } else {
                        // createdBy doit pouvoir être NULL pour OrderShop, en attendant qu'on en discute, j'assigne ça au premier de la base
                        $userRepository = $this->em->getRepository($this->em->getClassMetadata(UserInterface::class)->getName());
                        $user = $userRepository->find(1);
                        $orderShop->setCreatedBy($user);
                        $orderShop->setUpdatedBy($user);
                }

                $this->em->persist($orderShop);
                $this->em->flush();

                return $orderShop;
        }

        public function addOrderProduct($orderShop, $orderProductAdd)
        {
                if ($orderProductAdd->getQuantityOrder() > 0) {
                        $updated = false;

                        $orderProductAdd->setTitle($orderProductAdd->getTitleOrderShop());
                        $orderProductAdd->setPrice($this->priceUtils->getPrice($orderProductAdd->getProduct()));
                        $orderProductAdd->setUnit($orderProductAdd->getProduct()->getUnitInherited());
                        $orderProductAdd->setTaxRate($orderProductAdd->getProduct()->getTaxRateInherited());
                        $orderProductAdd->setQuantityProduct($orderProductAdd->getProduct()->getQuantityInherited());

                        $productFamily = $this->productFamilyUtils->getProductFamilyBySlug($orderProductAdd->getProduct()->getProductFamily()->getSlug());
                        $reductionCatalog = $productFamily->getReductionCatalog();
                        if ($reductionCatalog) {
                                $orderProductReductionCatalog = new OrderProductReductionCatalog();
                                $orderProductReductionCatalog->setTitle($reductionCatalog->getTitle());
                                $orderProductReductionCatalog->setValue($reductionCatalog->getValue());
                                $orderProductReductionCatalog->setUnit($reductionCatalog->getUnit());
                                $orderProductReductionCatalog->setBehaviorTaxRate($reductionCatalog->getBehaviorTaxRate());

                                $orderProductAdd->setOrderProductReductionCatalog($orderProductReductionCatalog);
                        }

                        foreach ($orderShop->getOrderProducts() as $orderProduct) {
                                if ($orderProduct->getProduct()->getId() == $orderProductAdd->getProduct()->getId()
                                        && $this->priceUtils->getPrice($orderProduct) == $this->priceUtils->getPrice($orderProductAdd)
                                        && $this->compareOrderProductReductionCatalog($orderProduct->getOrderProductReductionCatalog(), $orderProductAdd->getOrderProductReductionCatalog())) {

                                        $updated = true;
                                        $orderProduct->setQuantityOrder($orderProduct->getQuantityOrder() + $orderProductAdd->getQuantityOrder());
                                        $this->em->persist($orderProduct);
                                }
                        }

                        if (!$updated) {
                                $orderShop->addOrderProduct($orderProductAdd);

                                if (isset($orderProductReductionCatalog)) {
                                        $this->em->persist($orderProductReductionCatalog);
                                }

                                $this->em->persist($orderProductAdd);
                                $this->em->persist($orderShop);
                        }

                        $this->em->flush();
                }

        }

        public function compareOrderProductReductionCatalog($orderProductReductionCatalog1, $orderProductReductionCatalog2)
        {
                return $orderProductReductionCatalog1 && $orderProductReductionCatalog2
                        && $orderProductReductionCatalog1->getUnit() == $orderProductReductionCatalog2->getUnit()
                        && $orderProductReductionCatalog1->getValue() == $orderProductReductionCatalog2->getValue()
                        && $orderProductReductionCatalog1->getBehaviorTaxRate() == $orderProductReductionCatalog2->getBehaviorTaxRate();
        }

        public function countQuantities($orderShop)
        {
                return $this->countQuantitiesByOrderProducts($orderShop->getOrderProducts());
        }

        public function countQuantitiesByOrderProducts($orderProducts = [])
        {
                $count = 0;

                foreach ($orderProducts as $orderProduct) {
                        $count += $orderProduct->getQuantityOrder();
                }

                return $count;
        }

        public function getOrderProductsByParentCategory($orderShop = null)
        {
                $categoriesArray = [];
                foreach ($orderShop->getOrderProducts() as $orderProduct) {
                        $productCategories = $orderProduct->getProduct()->getProductFamily()->getProductCategories();
                        $category = $productCategories[0]->getParentCategory();
                        $labelCategory = $category->getTitle();
                        if (!isset($categoriesArray[$labelCategory])) {
                                $categoriesArray[$labelCategory] = [];
                        }
                        $categoriesArray[$labelCategory][] = $orderProduct;
                }

                return $categoriesArray;
        }

        public function getOrderDatas($order = null)
        {
                if (!$order) {
                        $order = $this->getCartCurrent();
                }

                $data = [];
                $data['order'] = $order;
                $data['count'] = $this->countQuantities($order);
                $data['total_with_tax'] = $this->priceUtils->getTotalWithTaxAndReduction($order);
                $data['order_products_by_category'] = $this->getOrderProductsByParentCategory($order);

                return $data;
        }


        public function getOrderAsJsonObject(OrderShopInterface $order)
        {
                $data['id'] = $order->getId();
                $data['user'] = $order->getUser()->getSummary();
                $data['orderStatus'] = $order->getOrderStatus()->__tosString();
                $data['deliveryAddress'] = $order->getDeliveryAddress()->getSummary();
                $data['invoiceAddress'] = $order->getInvoiceAddress()->getSummary();
                $data['total'] = $this->priceUtils->getTotal($order);
                $data['totalWithTax'] = $this->priceUtils->getTotalWithTax($order);
                $data['totalWithTaxAndReduction'] = $this->priceUtils->getTotalWithTax($order);
                $i = 0;
                foreach ($this->getOrderProductsByParentCategory($order) as $labelCategory => $orderProducts) {

                        foreach ($orderProducts as $orderProduct) {
                                $data['orderProducts'][$i]['id'] = $orderProduct->getId();
                                $data['orderProducts'][$i]['product'] = $orderProduct->getProduct()->getId();
                                $data['orderProducts'][$i]['quantityOrder'] = $orderProduct->getQuantityOrder();
                                $data['orderProducts'][$i]['labelCategory'] = $labelCategory;
                                $data['orderProducts'][$i]['title'] = $orderProduct->getTitle();
                                $data['orderProducts'][$i]['price'] = $this->priceUtils->getPrice($orderProduct);
                                $data['orderProducts'][$i]['priceWithTax'] = $this->priceUtils->getPriceWithTax($orderProduct);
                                $data['orderProducts'][$i]['priceWithTaxAndReduction'] = $this->priceUtils->getPriceWithTaxAndReduction($orderProduct);
                                $data['orderProducts'][$i]['quantity'] = $orderProduct->getQuantityOrder();
                                $data['orderProducts'][$i]['totalWithTaxAndReduction'] = $this->priceUtils->getTotalOrderProductsWithTaxAndReduction(array($orderProduct));
                                $i++;
                        }
                }

                return $data;
        }

        public function getSummaryOrderProductReductionCatalog($orderProductReductionCatalog)
        {
                $text = '';

                if ($orderProductReductionCatalog) {
                        if ($orderProductReductionCatalog->getUnit() == 'amount') {
                                $text .= '- ' . $orderProductReductionCatalog->getValue() . '&nbsp;€';
                        }

                        if ($orderProductReductionCatalog->getUnit() == 'percent') {
                                $text .= '- ' . $orderProductReductionCatalog->getValue() . '&nbsp;%';
                        }
                }

                return $text;
        }

        public function newOrderStatusHistory($order, $status, $origin = 'user')
        {
                $orderStatusHistoryClass = $this->em->getClassMetadata(OrderStatusHistoryInterface::class);
                $orderStatusHistory = new $orderStatusHistoryClass->name;
                $orderStatusHistory->setOrderShop($order);
                $orderStatusHistory->setOrderStatus($status);
                $orderStatusHistory->setOrigin($origin);
                $this->em->persist($orderStatusHistory);
        }


        public function createOrderReductionCart(OrderShopInterface $orderShop, ReductionCartInterface $reductionCart)
        {
                $orderReductionCartClass = $this->em->getClassMetadata(OrderReductionCartInterface::class);
                $orderReductionCart = new $orderReductionCartClass->name;

                $orderReductionCart->setOrderShop($orderShop);
                $orderReductionCart->setReductionCart($reductionCart);

                $orderReductionCart->setTitle($reductionCart->getTitle());
                $orderReductionCart->setValue($reductionCart->getValue());
                $orderReductionCart->setUnit($reductionCart->getUnit());
                $orderReductionCart->setBehaviorTaxRate($reductionCart->getBehaviorTaxRate());
                $orderReductionCart->setFreeShipping($reductionCart->getFreeShipping());
                $orderReductionCart->setAppliedTo($reductionCart->getAppliedTo());
                $orderReductionCart->setType($reductionCart->getType());


                return $orderReductionCart;
        }

        /*public function getReductionCreditsAvailable($order)
        {
                $reductionCreditRepo = $this->em->getRepository(ReductionCreditInterface::class);

                $reductionCredits = $reductionCreditRepo->getReductionCreditByUser($order->getUser());
                foreach ($reductionCredits as $reductionCredit){

                }
        }*/
}